// **********************************************************************
//
// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

// Ice version 3.3.1

// Freeze types in this file:
// name="IceStorm::SubscriberMap", key="IceStorm::SubscriberRecordKey", value="IceStorm::SubscriberRecord"

#include <Ice/BasicStream.h>
#include <IceUtil/StringUtil.h>
#include <IceStorm/SubscriberMap.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 303
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 1
#       error Ice patch level mismatch!
#   endif
#endif

void
IceStorm::SubscriberMapKeyCodec::write(const ::IceStorm::SubscriberRecordKey& v, Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    v.__write(&stream);
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::SubscriberMapKeyCodec::read(::IceStorm::SubscriberRecordKey& v, const Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    v.__read(&stream);
}

static const ::std::string __IceStorm__SubscriberMapKeyCodec_typeId = "::IceStorm::SubscriberRecordKey";

const ::std::string&
IceStorm::SubscriberMapKeyCodec::typeId()
{
    return __IceStorm__SubscriberMapKeyCodec_typeId;
}

void
IceStorm::SubscriberMapValueCodec::write(const ::IceStorm::SubscriberRecord& v, Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.startWriteEncaps();
    v.__write(&stream);
    stream.endWriteEncaps();
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::SubscriberMapValueCodec::read(::IceStorm::SubscriberRecord& v, const Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    stream.startReadEncaps();
    v.__read(&stream);
    stream.endReadEncaps();
}

static const ::std::string __IceStorm__SubscriberMapValueCodec_typeId = "::IceStorm::SubscriberRecord";

const ::std::string&
IceStorm::SubscriberMapValueCodec::typeId()
{
    return __IceStorm__SubscriberMapValueCodec_typeId;
}
