// **********************************************************************
//
// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

// Ice version 3.3.1

// Freeze types in this file:
// name="IceStorm::V31Format", key="string", value="IceStorm::LinkRecordDict"

#include <Ice/BasicStream.h>
#include <IceUtil/StringUtil.h>
#include <IceStorm/V31FormatDB.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 303
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 1
#       error Ice patch level mismatch!
#   endif
#endif

void
IceStorm::V31FormatKeyCodec::write(const ::std::string& v, Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.write(v);
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::V31FormatKeyCodec::read(::std::string& v, const Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    stream.read(v);
}

static const ::std::string __IceStorm__V31FormatKeyCodec_typeId = "string";

const ::std::string&
IceStorm::V31FormatKeyCodec::typeId()
{
    return __IceStorm__V31FormatKeyCodec_typeId;
}

void
IceStorm::V31FormatValueCodec::write(const ::IceStorm::LinkRecordDict& v, Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.startWriteEncaps();
    ::IceStorm::__writeLinkRecordDict(&stream, v);
    stream.endWriteEncaps();
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::V31FormatValueCodec::read(::IceStorm::LinkRecordDict& v, const Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    stream.startReadEncaps();
    ::IceStorm::__readLinkRecordDict(&stream, v);
    stream.endReadEncaps();
}

static const ::std::string __IceStorm__V31FormatValueCodec_typeId = "::IceStorm::LinkRecordDict";

const ::std::string&
IceStorm::V31FormatValueCodec::typeId()
{
    return __IceStorm__V31FormatValueCodec_typeId;
}
